searchNodes=[{"doc":"Handy little queues and producers.","ref":"Queutils.html","title":"Queutils","type":"module"},{"doc":"A GenStage producer that acts as a blocking queue, with a fixed length. Blocks any time Queutils.BlockingProducer.push/2 is called when the queue is at its maximum length.This can be used as an entry-point to a GenStage pipeline, since the max queue length provides for back-pressure. You can even set the queue's length to zero in order to block all pushes until demand comes in.UsageAdd it to your application supervisor's start/2 function like this:def start(_type, _args) do children = [ ... {Queutils.BlockingProducer, name: MessageProducer, max_length: 10_000}, ... ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endThen, subscribe a GenStage to it.def init(:ok) do {:consumer, :the_state_does_not_matter, subscribe_to: [MessageProducer]} end ``` You can now push messages to the queue like this: :ok = Queutils.BlockingProducer.push(MessageProducer, :my_message) Broadway users be forewared! A Broadway module needs to start its producer itself, so it&#39;s not possible to customize the process ID a la the `:name` option documented below. If you&#39;re in that boat, you should use a `Queutils.BlockingQueue` along with a `Queutils.BlockingQueueProducer`, so you can customize your reference to your `BlockingQueue`. ## Options - `:name` - the ID of the queue. This will be the first argument to the `push/2` function. - `:max_length` - The maximum number of messages that this process will store until it starts blocking. Default is 1,000. - `:dispatcher` - The `GenStage` dispatcher that this producer should use. Default is `GenStage.DemandDispatcher`.","ref":"Queutils.BlockingProducer.html","title":"Queutils.BlockingProducer","type":"module"},{"doc":"Get the number of items currently sitting in a queue.","ref":"Queutils.BlockingProducer.html#length/1","title":"Queutils.BlockingProducer.length/1","type":"function"},{"doc":"Get the total count of messages that have been popped from a queue over its lifetime.","ref":"Queutils.BlockingProducer.html#popped_count/1","title":"Queutils.BlockingProducer.popped_count/1","type":"function"},{"doc":"Push an item onto the queue. This function will block if the queue is full, and unblock once it's not.","ref":"Queutils.BlockingProducer.html#push/2","title":"Queutils.BlockingProducer.push/2","type":"function"},{"doc":"Get the total count of messages that have been pushed to a queue over its lifetime.","ref":"Queutils.BlockingProducer.html#pushed_count/1","title":"Queutils.BlockingProducer.pushed_count/1","type":"function"},{"doc":"","ref":"Queutils.BlockingProducer.html#start_link/1","title":"Queutils.BlockingProducer.start_link/1","type":"function"},{"doc":"Get the number of blocked processes waiting to push to a queue.","ref":"Queutils.BlockingProducer.html#waiting_count/1","title":"Queutils.BlockingProducer.waiting_count/1","type":"function"},{"doc":"A queue with a fixed length that blocks on Queutils.BlockingQueue.push/2 if the queue is full.UsageAdd it to your application supervisor's start/2 function, like this:def start(_type, _args) do children = [ ... {Queutils.BlockingQueue, name: MessageQueue, max_length: 10_000}, ... ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endThen you can push and pop from the queue like this::ok = Queutils.Blockingqueue.push(MessageQueue, :my_message) [:my_message] = Queutils.Blockingqueue.pop(MessageQueue, 1)Use with Queutils.BlockingQueueProducer for more fun, or use Queutils.BlockingProducer for the effect of both.Options:name - the ID of the queue. This will be the first argument to the push/2 function. Default is BlockingQueue.:max_length - The maximum number of messages that this process will store until it starts blocking. Default is 1,000.","ref":"Queutils.BlockingQueue.html","title":"Queutils.BlockingQueue","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.See Supervisor.","ref":"Queutils.BlockingQueue.html#child_spec/1","title":"Queutils.BlockingQueue.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.init/1.","ref":"Queutils.BlockingQueue.html#init/1","title":"Queutils.BlockingQueue.init/1","type":"function"},{"doc":"Get the current length of the queue.","ref":"Queutils.BlockingQueue.html#length/1","title":"Queutils.BlockingQueue.length/1","type":"function"},{"doc":"Pop an item off of the queue. Never blocks, and returns a list. The returned list will be empty if the queue is empty.","ref":"Queutils.BlockingQueue.html#pop/2","title":"Queutils.BlockingQueue.pop/2","type":"function"},{"doc":"Get the count of elements that have been popped from a queue over the queue's lifetime.","ref":"Queutils.BlockingQueue.html#popped_count/1","title":"Queutils.BlockingQueue.popped_count/1","type":"function"},{"doc":"Push an item onto the queue. This function will block if the queue is full, and unblock once it's not.","ref":"Queutils.BlockingQueue.html#push/2","title":"Queutils.BlockingQueue.push/2","type":"function"},{"doc":"Get the count of elements that have been pushed to a queue over the queue's lifetime.","ref":"Queutils.BlockingQueue.html#pushed_count/1","title":"Queutils.BlockingQueue.pushed_count/1","type":"function"},{"doc":"Start a blocking queue process.Options:name - the ID of the queue. This will be the first argument to the push/2 function. Default is BlockingQueue.:max_length - The maximum number of messages that this process will store until it starts blocking. Default is 1,000.","ref":"Queutils.BlockingQueue.html#start_link/1","title":"Queutils.BlockingQueue.start_link/1","type":"function"},{"doc":"A GenStage producer that polls a Queutils.BlockingQueue at a fixed interval, emitting any events on the queue.UsageAdd it to your application supervisor's start/2 function, after the queue it pulls from, like this:def start(_type, _args) do children = [ ... {Queutils.BlockingQueue, name: MessageQueue, max_length: 10_000}, {Queutils.BlockingQueueProducer, name: MessageProducer}, ... ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endThe subscribe a consumer to it, like any other GenStage producer.def init(_opts) do {:consumer, :my_consumer_state, [subscribe_to: MessageProducer]} endOptions:name - the ID of the queue. This will be the first argument to the push/2 function. Default is BlockingQueueProducer.:max_length - The maximum number of messages that this process will store until it starts blocking. Default is 1,000.:dispatcher - The GenStage dispatcher that this producer should use. Default is GenStage.DemandDispatcher.","ref":"Queutils.BlockingQueueProducer.html","title":"Queutils.BlockingQueueProducer","type":"module"},{"doc":"Start a blocking queue producer.Options:name - the ID of the queue. This will be the first argument to the push/2 function. Default is BlockingQueueProducer.:max_length - The maximum number of messages that this process will store until it starts blocking. Default is 1,000.:dispatcher - The GenStage dispatcher that this producer should use. Default is GenStage.DemandDispatcher.","ref":"Queutils.BlockingQueueProducer.html#start_link/1","title":"Queutils.BlockingQueueProducer.start_link/1","type":"function"},{"doc":"Queutils Handy little queues and producers that make using GenStage or Broadway a breeze.Getting events into a GenStage or Broadway pipeline is more difficult than it seems at first glance. Any message producer based on GenStage needs to track demand, and emit events as it receives them if there is surplus demand. Also, any producer implementation needs to provide back-pressure, a key part of GenStage and Broadway's design. That's why a standalone blocking queue process is most the most ideal producer for these libraries.Decouples message producers and consumers, so producers do not need to wait for the consumer to be readyBlocks when full, so producers don't keep producing messages if the consumer is overwhelmedTracks demand, and immediately emits events if demand is pendingThis library also provides a blocking queue implementation that can be used in pure Elixir.","ref":"readme.html","title":"Queutils","type":"extras"},{"doc":"This library is available in Hex, and the package can be installed by adding queutils to your list of dependencies in mix.exs:def deps do [ {:queutils, &quot;~&gt; 1.2&quot;} ] endDocumentation can be generated with ExDoc and can be found online at https://hexdocs.pm/queutils.","ref":"readme.html#installation","title":"Queutils - Installation","type":"extras"},{"doc":"If you just want a queue to communicate between processes, use a Queutils.BlockingQueue. This module implements a queue with a fixed length, and any calls to Queutils.BlockingQueue.push/2 will block until the queue has room again.If you're working with GenStage, you probably want to use Queutils.BlockingProducer. This module is just like a Queutils.BlockingQueue, but it also provides callbacks that let a GenStage consumer subscribe to it. This way you can push messages into a GenStage pipeline.Lastly, a Queutils.BlockingQueueProducer acts just like a Queutils.BlockingProducer, except you need to provide the queue yourself. This is the module to use if you're working with Broadway, because Broadway stages needs to start up their producers themselves.With plain ElixirAdd Queutils.BlockingQueue to your application supervisor's start/2 function, like this:def start(_type, _args) do children = [ {Queutils.BlockingQueue, name: MessageQueue, max_length: 10_000}, ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endYou can now push messages to the queue like this::ok = Queutils.BlockingQueue.push(MessageQueue, :my_message)and pop from it like this::my_message = Queutils.BlockingQueue.pop(MessageQueue)With GenStageAdd Queutils.BlockingProducer to your application supervisor's start/2 function, like this:def start(_type, _args) do children = [ {Queutils.BlockingProducer, name: MessageProducer, max_length: 10_000} ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endThen, subscribe a GenStage to it.def init(:ok) do {:consumer, :the_state_does_not_matter, subscribe_to: [MessageProducer]} endYou can now push messages to the queue like this::ok = Queutils.BlockingProducer.push(MessageProducer, :my_message)With BroadwayAdd Queutils.BlockingQueue to your application supervisor's start/2 function, just like we're using it with plain Elixir:def start(_type, _args) do children = [ {Queutils.BlockingQueue, name: MessageQueue, max_length: 10_000}, ] opts = [strategy: :one_for_one, name: MyApplication.Supervisor] Supervisor.start_link(children, opts) endThen, add a Queutils.BlockingQueueProducer as your Broadway stage's producer, pointing it to the queue you just created.def start_link(_opts) do Broadway.start_link(__MODULE__, name: __MODULE__, producer: [ module: {Queutils.BlockingQueueProducer, queue: MessageQueue}, transformer: {MyApplication.Transformer, :transform, []} ], processors: [ default: [] ] ) endYou will need to add a :transformer option to your Broadway stage in order to wrap messages in a Broadway.Message struct. It's easy, but needs to be done. See Broadway's Custom Producers documentation for details.You can now push to the queue like this, and your Broadway stage will pick it up::ok = Queutils.Blockingqueue.push(MessageQueue, :my_message)","ref":"readme.html#usage","title":"Queutils - Usage","type":"extras"},{"doc":"This project was developed by Rosa Richter. You can get in touch with her on Keybase.io.","ref":"readme.html#maintainer","title":"Queutils - Maintainer","type":"extras"},{"doc":"Questions and pull requests are more than welcome. I follow Elixir's tenet of bad documentation being a bug, so if anything is unclear, please file an issue! Ideally, my answer to your question will be in an update to the docs.Please see CONTRIBUTING.md for all the details you could ever want about helping me with this project.Note that this project is released with a Contributor Code of Conduct. By participating in this project you agree to abide by its terms.","ref":"readme.html#contributing","title":"Queutils - Contributing","type":"extras"},{"doc":"MIT LicenseCopyright 2020 Rosa Richter.Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"readme.html#license","title":"Queutils - License","type":"extras"}]